@@ -47,6 +47,7 @@
 #include "nact-iactions-list.h"
 #include "nact-clipboard.h"
 #include "nact-main-statusbar.h"
+#include "nact-main-toolbar.h"
 #include "nact-main-tab.h"
 #include "nact-main-menubar.h"
 
@@ -59,6 +60,13 @@
 #define MENUBAR_IPREFS_TOOLS_TOOLBAR		"main-tools-toolbar"
 #define MENUBAR_IPREFS_HELP_TOOLBAR			"main-help-toolbar"
 
+enum {
+	MENUBAR_FILE_TOOLBAR_POS = 0,
+	MENUBAR_EDIT_TOOLBAR_POS,
+	MENUBAR_TOOLS_TOOLBAR_POS,
+	MENUBAR_HELP_TOOLBAR_POS
+};
+
 /* GtkActivatable
  * gtk_action_get_tooltip are only available starting with Gtk 2.16
  * until this is a required level, we must have some code to do the
@@ -128,6 +136,7 @@ static void     on_view_file_toolbar_activated( GtkToggleAction *action, NactMai
 static void     on_view_edit_toolbar_activated( GtkToggleAction *action, NactMainWindow *window );
 static void     on_view_tools_toolbar_activated( GtkToggleAction *action, NactMainWindow *window );
 static void     on_view_help_toolbar_activated( GtkToggleAction *action, NactMainWindow *window );
+static void     on_view_toolbar_activated( GtkToggleAction *action, NactMainWindow *window, int toolbar_id );
 
 static void     on_import_activated( GtkAction *action, NactMainWindow *window );
 static void     on_export_activated( GtkAction *action, NactMainWindow *window );
@@ -148,8 +157,6 @@ static void     on_menu_item_deselected( GtkMenuItem *proxy, NactMainWindow *win
 static void     on_popup_selection_done(GtkMenuShell *menushell, NactMainWindow *window );
 static void     on_proxy_connect( GtkActionGroup *action_group, GtkAction *action, GtkWidget *proxy, NactMainWindow *window );
 static void     on_proxy_disconnect( GtkActionGroup *action_group, GtkAction *action, GtkWidget *proxy, NactMainWindow *window );
-static void     on_view_toolbar_activated( GtkToggleAction *action, NactMainWindow *window, const gchar *pref, const gchar *path, int pos );
-static void     toolbar_init( NactMainWindow *window, const gchar *pref, gboolean default_value, const gchar *path, const gchar *item );
 
 static const GtkActionEntry entries[] = {
 
@@ -205,9 +212,9 @@ static const GtkActionEntry entries[] = {
 				/* i18n: tooltip displayed in the status bar when selecting the Delete item */
 				N_( "Delete the selected item(s)" ),
 				G_CALLBACK( on_delete_activated ) },
-		{ "ReloadActionsItem", GTK_STOCK_REFRESH, N_( "_Reload the list of actions" ), "F5",
+		{ "ReloadActionsItem", GTK_STOCK_REFRESH, N_( "_Reload the items" ), "F5",
 				/* i18n: tooltip displayed in the status bar when selecting the 'Reload' item */
-				N_( "Cancel your current modifications and reload the list of actions" ),
+				N_( "Cancel your current modifications and reload the initial list of menus and actions" ),
 				G_CALLBACK( on_reload_activated ) },
 		{ "PreferencesItem", GTK_STOCK_PREFERENCES, NULL, NULL,
 				/* i18n: tooltip displayed in the status bar when selecting the 'Preferences' item */
@@ -412,10 +419,7 @@ nact_main_menubar_runtime_init( NactMainWindow *window )
 	mis = g_new0( MenubarIndicatorsStruct, 1 );
 	g_object_set_data( G_OBJECT( window ), MENUBAR_PROP_INDICATORS, mis );
 
-	toolbar_init( window, MENUBAR_IPREFS_FILE_TOOLBAR, TRUE, "/ui/FileToolbar", "ViewFileToolbarItem" );
-	toolbar_init( window, MENUBAR_IPREFS_EDIT_TOOLBAR, FALSE, "/ui/EditToolbar", "ViewEditToolbarItem" );
-	toolbar_init( window, MENUBAR_IPREFS_TOOLS_TOOLBAR, FALSE, "/ui/ToolsToolbar", "ViewToolsToolbarItem" );
-	toolbar_init( window, MENUBAR_IPREFS_HELP_TOOLBAR, FALSE, "/ui/HelpToolbar", "ViewHelpToolbarItem" );
+	nact_main_toolbar_init( window, action_group );
 }
 
 /**
@@ -524,7 +528,7 @@ on_iactions_list_selection_changed( NactMainWindow *window, GList *selected )
 {
 	MenubarIndicatorsStruct *mis;
 
-	g_debug( "nact_main_menubar_on_iactions_list_selection_changed: selected=%p (%d)",
+	g_debug( "nact_main_menubar_on_iactions_list_selection_changed: selected=%p (count=%d)",
 			( void * ) selected, g_list_length( selected ));
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
@@ -591,123 +595,219 @@ static void
 on_update_sensitivities( NactMainWindow *window, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_main_menubar_on_update_sensitivities";
+	NactApplication *application;
+	NAPivot *pivot;
+	gboolean is_level_zero_writable;
+	gboolean has_writable_providers;
+	gboolean has_modified_items;
+	GList *selected_items, *is;
+	NAObject *first_parent;
+	gboolean is_first_parent_writable;
+	gboolean new_item_enabled;
+	gboolean new_profile_enabled;
+	NAObject *selected_action;
+	NAObject *parent_item;
 	MenubarIndicatorsStruct *mis;
-	NAObject *item;
-	NAObject *profile;
-	NAObject *selected_row;
-	gboolean has_modified;
-	gint count_list;
+	gboolean is_clipboard_empty;
 	gint count_selected;
+	gboolean are_parents_writable;
+	NAObject *selected_item;
 	gboolean cut_enabled;
 	gboolean copy_enabled;
 	gboolean paste_enabled;
 	gboolean paste_into_enabled;
 	gboolean duplicate_enabled;
 	gboolean delete_enabled;
-	gboolean clipboard_is_empty;
-	gboolean new_item_enabled;
-	gboolean readonly_item;
-	gboolean writable_provider;
-	gboolean writable_item;
-	gboolean has_writables;
-	NAObjectItem *parent;
-	gboolean writable_parent;
+	gint count_list;
 
-	g_debug( "%s: window=%p", thisfn, ( void * ) window );
+	g_debug( "%s: window=%p, user_data=%p", thisfn, ( void * ) window, ( void * ) user_data );
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+	pivot = nact_application_get_pivot( application );
+	is_level_zero_writable = na_pivot_is_level_zero_writable( pivot );
+
+	has_writable_providers = nact_window_has_writable_providers( NACT_WINDOW( window ));
+	has_modified_items = nact_main_window_has_modified_items( window );
+
+	selected_items = nact_iactions_list_bis_get_selected_items( NACT_IACTIONS_LIST( window ));
+	first_parent = selected_items && g_list_length( selected_items )
+			? ( NAObject * ) na_object_get_parent( selected_items->data )
+			: NULL;
+	is_first_parent_writable = first_parent
+			? nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( first_parent ))
+			: is_level_zero_writable;
+
 	mis = ( MenubarIndicatorsStruct * ) g_object_get_data( G_OBJECT( window ), MENUBAR_PROP_INDICATORS );
-	has_writables = nact_window_has_writable_providers( NACT_WINDOW( window ));
+	is_clipboard_empty = ( mis->clipboard_menus + mis->clipboard_actions + mis->clipboard_profiles == 0 );
+	count_selected = selected_items ? g_list_length( selected_items ) : 0;
+	g_debug( "%s: count_selected=%d", thisfn, count_selected );
 
-	g_object_get(
-			G_OBJECT( window ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, &item,
-			TAB_UPDATABLE_PROP_EDITED_PROFILE, &profile,
-			TAB_UPDATABLE_PROP_SELECTED_ROW, &selected_row,
-			NULL );
-	g_return_if_fail( !item || NA_IS_OBJECT_ITEM( item ));
-	g_return_if_fail( !profile || NA_IS_OBJECT_PROFILE( profile ));
-
-	has_modified = nact_main_window_has_modified_items( window );
-	readonly_item = item ? na_object_is_readonly( item ) : FALSE;
-	writable_provider = item ? nact_window_is_writable_provider( NACT_WINDOW( window ), NA_OBJECT_ITEM( item )) : FALSE;
-	writable_item = writable_provider && !readonly_item;
-
-	parent = item ? na_object_get_parent( item ) : NULL;
-	writable_parent = parent
-		? nact_window_is_writable_provider( NACT_WINDOW( window ), parent ) && !na_object_is_readonly( parent )
-		: FALSE;
-
-	/* new menu enabled if selection is a menu or an action
-	 * new action enabled if selection is a menu or an action
-	 * adding a new item requires that parent be writable
+	/* new menu / new action
+	 * new item will be inserted just before beginning of selection
+	 * parent of the first selected row must be writable
+	 * we must have at least one writable provider
 	 */
-	new_item_enabled = ( selected_row == NULL || NA_IS_OBJECT_ITEM( selected_row ));
-	new_item_enabled = new_item_enabled && has_writables;
-	new_item_enabled = new_item_enabled && ( parent ? writable_parent : TRUE );
+	new_item_enabled = is_first_parent_writable && has_writable_providers;
 	enable_item( window, "NewMenuItem", new_item_enabled );
 	enable_item( window, "NewActionItem", new_item_enabled );
 
-	/* new profile enabled if selection is relative to only one writable action */
-	enable_item( window, "NewProfileItem",
-			item != NULL && !NA_IS_OBJECT_MENU( item ) && writable_item );
+	/* new profile enabled if selection is relative to only one writable action
+	 * i.e. contains profile(s) of the same action, or only contains one action
+	 * action must be writable
+	 */
+	new_profile_enabled = TRUE;
+	selected_action = NULL;
+	for( is = selected_items ; is ; is = is->next ){
 
-	/* save enabled if at least one item has been modified */
-	enable_item( window, "SaveItem", has_modified || mis->level_zero_order_changed );
+		if( NA_IS_OBJECT_MENU( is->data )){
+			new_profile_enabled = FALSE;
+			break;
+
+		} else if( NA_IS_OBJECT_ACTION( is->data )){
+			if( !selected_action ){
+				selected_action = NA_OBJECT( is->data );
+			} else if( selected_action != is->data ){
+				new_profile_enabled = FALSE;
+				break;
+			}
+
+		} else if( NA_IS_OBJECT_PROFILE( is->data )){
+			parent_item = NA_OBJECT( na_object_get_parent( is->data ));
+			if( !selected_action ){
+				selected_action = parent_item;
+			} else if( selected_action != parent_item ){
+				new_profile_enabled = FALSE;
+				break;
+			}
+		}
+	}
+	enable_item( window, "NewProfileItem",
+			new_profile_enabled &&
+			selected_action != NULL &&
+			nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( selected_action )));
+
+	/* save enabled if at least one item has been modified
+	 * or level-zero has been resorted and is writable
+	 */
+	enable_item( window, "SaveItem",
+			has_modified_items || ( mis->level_zero_order_changed && is_level_zero_writable ));
 
 	/* quit always enabled */
 
-	/* edit menu is only enabled when the treeview has the focus
+	/* cut requires a non-empty selection
+	 * and that all parents are writable (as implies a delete operation)
 	 */
+	cut_enabled = mis->treeview_has_focus || mis->popup_handler;
+	cut_enabled &= count_selected > 0;
+	are_parents_writable = TRUE;
+	for( is = selected_items ; is ; is = is->next ){
+		parent_item = ( NAObject * ) na_object_get_parent( is->data );
+		if( parent_item ){
+			if( !nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( parent_item ))){
+				are_parents_writable = FALSE;
+				break;
+			}
+		} else if( !is_level_zero_writable ){
+			are_parents_writable = FALSE;
+			break;
+		}
+	}
+	cut_enabled &= are_parents_writable;
+	enable_item( window, "CutItem", cut_enabled );
 
-	clipboard_is_empty = ( mis->clipboard_menus + mis->clipboard_actions + mis->clipboard_profiles == 0 );
-	count_selected = mis->selected_menus + mis->selected_actions + mis->selected_profiles;
-
-	/* cut/copy/duplicate/delete enabled when selection not empty */
-	/* cut/delete require a writable item */
-	cut_enabled = ( mis->treeview_has_focus || mis->popup_handler ) && count_selected > 0 && writable_item && has_writables;
-	copy_enabled = ( mis->treeview_has_focus || mis->popup_handler ) && count_selected > 0 && has_writables;
-	duplicate_enabled = ( mis->treeview_has_focus || mis->popup_handler ) && count_selected > 0 && has_writables;
-	delete_enabled = ( mis->treeview_has_focus || mis->popup_handler ) && count_selected > 0 && writable_item;
+	/* copy only requires a non-empty selection */
+	copy_enabled = mis->treeview_has_focus || mis->popup_handler;
+	copy_enabled &= count_selected > 0;
+	enable_item( window, "CopyItem", copy_enabled );
 
 	/* paste enabled if
-	 * - simple selection
-	 * - clipboard contains only profiles, and current selection is a profile
-	 * - clipboard contains actions or menus, and current selection is a menu or an action */
-	paste_enabled = FALSE;
-	if(( mis->treeview_has_focus || mis->popup_handler ) && count_selected <= 1 ){
-		if( !clipboard_is_empty ){
-			if( mis->clipboard_profiles ){
-				paste_enabled = item && NA_IS_OBJECT_ACTION( item ) && writable_item;
-			} else {
-				paste_enabled = ( item != NULL ) && has_writables && ( parent ? writable_parent : TRUE );
+	 * - clipboard is not empty
+	 * - current selection is not multiple
+	 * - if clipboard contains only profiles,
+	 *   then current selection must be a profile or an action
+	 *   and the action must be writable
+	 * - if clipboard contains actions or menus,
+	 *   then current selection (if any) must be a menu or an action
+	 *   and its parent must be writable
+	 */
+	paste_enabled = mis->treeview_has_focus || mis->popup_handler;
+	paste_enabled &= !is_clipboard_empty;
+	paste_enabled &= count_selected <= 1;
+	if( mis->clipboard_profiles ){
+		paste_enabled &= count_selected == 1;
+		if( paste_enabled ){
+			selected_action = NA_OBJECT(
+					NA_IS_OBJECT_PROFILE( selected_items->data )
+							? na_object_get_parent( selected_items->data )
+							: selected_items->data );
+			paste_enabled &= NA_IS_OBJECT_ACTION( selected_action );
+			paste_enabled &= nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( selected_action ));
+		}
+	} else {
+		paste_enabled &= has_writable_providers;
+		if( count_selected ){
+			selected_item = NA_OBJECT( selected_items->data );
+			paste_enabled &= NA_IS_OBJECT_ITEM( selected_item );
+			if( paste_enabled ){
+				parent_item = ( NAObject * ) na_object_get_parent( selected_item );
+				paste_enabled &= parent_item
+						? nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( parent_item ))
+						: is_level_zero_writable;
 			}
+		} else {
+			paste_enabled &= is_level_zero_writable;
 		}
 	}
+	enable_item( window, "PasteItem", paste_enabled );
 
 	/* paste into enabled if
-	 * - simple selection
-	 * - clipboard has profiles and current item is an action
-	 * - or current item is a menu
-	 * do not paste into if current selection is a profile */
-	paste_into_enabled = FALSE;
-	if(( mis->treeview_has_focus || mis->popup_handler ) && count_selected <= 1 ){
-		if( mis->selected_menus + mis->selected_actions ){
-			if( !clipboard_is_empty ){
-				if( mis->clipboard_profiles ){
-					paste_into_enabled = item && NA_IS_OBJECT_ACTION( item ) && writable_item;
-				} else {
-					paste_into_enabled = item && NA_IS_OBJECT_MENU( item ) && writable_item && has_writables;
-				}
+	 * - clipboard is not empty
+	 * - current selection is not multiple
+	 * - if clipboard contains only profiles,
+	 *   then current selection must be an action
+	 *   and the action must be writable
+	 * - if clipboard contains actions or menus,
+	 *   then current selection (if any) must be a menu
+	 *   and its parent must be writable
+	 */
+	paste_into_enabled = mis->treeview_has_focus || mis->popup_handler;
+	paste_into_enabled &= !is_clipboard_empty;
+	paste_into_enabled &= count_selected <= 1;
+	if( mis->clipboard_profiles ){
+		paste_enabled &= count_selected == 1;
+		if( paste_enabled ){
+			selected_action = NA_OBJECT( selected_items->data );
+			paste_enabled &= NA_IS_OBJECT_ACTION( selected_action );
+			paste_enabled &= nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( selected_action ));
+		}
+	} else {
+		paste_enabled &= has_writable_providers;
+		if( count_selected ){
+			selected_item = NA_OBJECT( selected_items->data );
+			paste_enabled &= NA_IS_OBJECT_MENU( selected_item );
+			if( paste_enabled ){
+				parent_item = ( NAObject * ) na_object_get_parent( selected_item );
+				paste_enabled &= parent_item
+						? nact_window_is_writable_item( NACT_WINDOW( window ), NA_OBJECT_ID( parent_item ))
+						: is_level_zero_writable;
 			}
+		} else {
+			paste_enabled &= is_level_zero_writable;
 		}
 	}
-
-	enable_item( window, "CutItem", cut_enabled );
-	enable_item( window, "CopyItem", copy_enabled );
-	enable_item( window, "PasteItem", paste_enabled );
 	enable_item( window, "PasteIntoItem", paste_into_enabled );
+
+	/* duplicate items will be duplicated besides each one
+	 * selection must be non-empty
+	 * each parent must be writable
+	 * -> so this is the same than cut
+	 */
+	duplicate_enabled = cut_enabled;
 	enable_item( window, "DuplicateItem", duplicate_enabled );
+
+	/* delete is same that cut */
+	delete_enabled = cut_enabled;
 	enable_item( window, "DeleteItem", delete_enabled );
 
 	/* reload items always enabled */
@@ -720,7 +820,7 @@ on_update_sensitivities( NactMainWindow *window, gpointer user_data )
 	enable_item( window, "CollapseAllItem", count_list > 0 );
 
 	/* import item enabled if at least one writable provider */
-	enable_item( window, "ImportItem", has_writables );
+	enable_item( window, "ImportItem", has_writable_providers );
 
 	/* export item enabled if IActionsList store contains actions */
 	enable_item( window, "ExportItem", mis->have_exportables );
@@ -729,6 +829,8 @@ on_update_sensitivities( NactMainWindow *window, gpointer user_data )
 	enable_item( window, "HelpItem", FALSE );
 
 	/* about always enabled */
+
+	na_object_free_items_list( selected_items );
 }
 
 static void
@@ -873,6 +975,8 @@ save_item( NactMainWindow *window, NAPivot *pivot, NAObjectItem *item )
 	GList *subitems, *it;
 	NAObjectItem *parent;
 	gint pos;
+	NAIIOProvider *provider_before;
+	NAIIOProvider *provider_after;
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 	g_return_if_fail( NA_IS_PIVOT( pivot ));
@@ -885,6 +989,8 @@ save_item( NactMainWindow *window, NAPivot *pivot, NAObjectItem *item )
 		}
 	}
 
+	provider_before = na_object_get_provider( item );
+
 	if( na_object_is_modified( item ) &&
 		nact_window_save_item( NACT_WINDOW( window ), item )){
 
@@ -921,6 +1027,11 @@ save_item( NactMainWindow *window, NAPivot *pivot, NAObjectItem *item )
 			} else {
 				na_pivot_add_item( pivot, dup_pivot );
 			}
+
+			provider_after = na_object_get_provider( item );
+			if( provider_after != provider_before ){
+				g_signal_emit_by_name( window, TAB_UPDATABLE_SIGNAL_PROVIDER_CHANGED, item );
+			}
 	}
 }
 
@@ -1226,25 +1337,41 @@ on_collapse_all_activated( GtkAction *gtk_action, NactMainWindow *window )
 static void
 on_view_file_toolbar_activated( GtkToggleAction *action, NactMainWindow *window )
 {
-	on_view_toolbar_activated( action, window, MENUBAR_IPREFS_FILE_TOOLBAR, "/ui/FileToolbar", 0 );
+	/*on_view_toolbar_activated( action, window, MENUBAR_IPREFS_FILE_TOOLBAR, "/ui/FileToolbar", MENUBAR_FILE_TOOLBAR_POS );*/
+	on_view_toolbar_activated( action, window, MAIN_TOOLBAR_FILE_ID );
 }
 
 static void
 on_view_edit_toolbar_activated( GtkToggleAction *action, NactMainWindow *window )
 {
-	on_view_toolbar_activated( action, window, MENUBAR_IPREFS_EDIT_TOOLBAR, "/ui/EditToolbar", 1 );
+	/*on_view_toolbar_activated( action, window, MENUBAR_IPREFS_EDIT_TOOLBAR, "/ui/EditToolbar", MENUBAR_EDIT_TOOLBAR_POS );*/
+	on_view_toolbar_activated( action, window, MAIN_TOOLBAR_EDIT_ID );
 }
 
 static void
 on_view_tools_toolbar_activated( GtkToggleAction *action, NactMainWindow *window )
 {
-	on_view_toolbar_activated( action, window, MENUBAR_IPREFS_TOOLS_TOOLBAR, "/ui/ToolsToolbar", 2 );
+	/*on_view_toolbar_activated( action, window, MENUBAR_IPREFS_TOOLS_TOOLBAR, "/ui/ToolsToolbar", MENUBAR_TOOLS_TOOLBAR_POS );*/
+	on_view_toolbar_activated( action, window, MAIN_TOOLBAR_TOOLS_ID );
 }
 
 static void
 on_view_help_toolbar_activated( GtkToggleAction *action, NactMainWindow *window )
 {
-	on_view_toolbar_activated( action, window, MENUBAR_IPREFS_HELP_TOOLBAR, "/ui/HelpToolbar", 3 );
+	/*on_view_toolbar_activated( action, window, MENUBAR_IPREFS_HELP_TOOLBAR, "/ui/HelpToolbar", MENUBAR_HELP_TOOLBAR_POS );*/
+	on_view_toolbar_activated( action, window, MAIN_TOOLBAR_HELP_ID );
+}
+
+static void
+on_view_toolbar_activated( GtkToggleAction *action, NactMainWindow *window, int toolbar_id )
+{
+	gboolean is_active;
+	GtkUIManager *ui_manager;
+
+	is_active = gtk_toggle_action_get_active( action );
+	ui_manager = ( GtkUIManager * ) g_object_get_data( G_OBJECT( window ), MENUBAR_PROP_UI_MANAGER );
+
+	nact_main_toolbar_activate( window, toolbar_id, ui_manager, is_active );
 }
 
 static void
@@ -1433,48 +1560,3 @@ on_proxy_disconnect( GtkActionGroup *action_group, GtkAction *action, GtkWidget
 {
 	/* signal handlers will be automagically disconnected on BaseWindow::dispose */
 }
-
-static void
-on_view_toolbar_activated( GtkToggleAction *action, NactMainWindow *window, const gchar *pref, const gchar *path, int pos )
-{
-	NactApplication *application;
-	NAPivot *pivot;
-	gboolean is_active;
-	GtkUIManager *ui_manager;
-	GtkWidget *hbox, *toolbar;
-
-	is_active = gtk_toggle_action_get_active( action );
-
-	ui_manager = ( GtkUIManager * ) g_object_get_data( G_OBJECT( window ), MENUBAR_PROP_UI_MANAGER );
-	toolbar = gtk_ui_manager_get_widget( ui_manager, path );
-	hbox = base_window_get_widget( BASE_WINDOW( window ), "ToolbarHBox" );
-	if( is_active ){
-		/*gtk_box_pack_start( GTK_BOX( hbox ), toolbar, FALSE, FALSE, 0 );*/
-		gtk_container_add( GTK_CONTAINER( hbox ), toolbar );
-		gtk_box_reorder_child( GTK_BOX( hbox ), toolbar, pos );
-	} else {
-		gtk_container_remove( GTK_CONTAINER( hbox ), toolbar );
-	}
-	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( application );
-	na_iprefs_write_bool( NA_IPREFS( pivot ), pref, is_active );
-}
-
-static void
-toolbar_init( NactMainWindow *window, const gchar *pref, gboolean default_value, const gchar *path, const gchar *item )
-{
-	NactApplication *application;
-	NAPivot *pivot;
-	gboolean is_active;
-	GtkActionGroup *group;
-	GtkToggleAction *action;
-
-	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	pivot = nact_application_get_pivot( application );
-	is_active = na_iprefs_read_bool( NA_IPREFS( pivot ), pref, default_value );
-	if( is_active ){
-		group = g_object_get_data( G_OBJECT( window ), MENUBAR_PROP_ACTIONS_GROUP );
-		action = GTK_TOGGLE_ACTION( gtk_action_group_get_action( group, item ));
-		gtk_toggle_action_set_active( action, TRUE );
-	}
-}
