@@ -32,6 +32,7 @@
 #include <config.h>
 #endif
 
+#include <gtk/gtk.h>
 #include <string.h>
 #include <uuid/uuid.h>
 
@@ -56,6 +57,7 @@ enum {
 	NAOBJECT_ITEM_PROP_ENABLED_ID,
 	NAOBJECT_ITEM_PROP_READONLY_ID,
 	NAOBJECT_ITEM_PROP_PROVIDER_ID,
+	NAOBJECT_ITEM_PROP_PROVIDER_DATA_ID,
 	NAOBJECT_ITEM_PROP_ITEMS_ID,
 };
 
@@ -64,6 +66,7 @@ enum {
 #define NAOBJECT_ITEM_PROP_ENABLED				"na-object-item-enabled"
 #define NAOBJECT_ITEM_PROP_READONLY				"na-object-item-read-only"
 #define NAOBJECT_ITEM_PROP_PROVIDER				"na-object-item-provider"
+#define NAOBJECT_ITEM_PROP_PROVIDER_DATA		"na-object-item-provider-data"
 #define NAOBJECT_ITEM_PROP_ITEMS				"na-object-item-items"
 
 static NAObjectIdClass *st_parent_class = NULL;
@@ -174,6 +177,13 @@ class_init( NAObjectItemClass *klass )
 			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
 	g_object_class_install_property( object_class, NAOBJECT_ITEM_PROP_PROVIDER_ID, spec );
 
+	spec = g_param_spec_pointer(
+			NAOBJECT_ITEM_PROP_PROVIDER_DATA,
+			"Provider data",
+			"Data specific to the provider",
+			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, NAOBJECT_ITEM_PROP_PROVIDER_DATA_ID, spec );
+
 	klass->private = g_new0( NAObjectItemClassPrivate, 1 );
 
 	naobject_class = NA_OBJECT_CLASS( klass );
@@ -211,6 +221,7 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	self->private->enabled = TRUE;
 	self->private->read_only = FALSE;
 	self->private->provider = NULL;
+	self->private->provider_data = NULL;
 }
 
 static void
@@ -244,6 +255,10 @@ instance_get_property( GObject *object, guint property_id, GValue *value, GParam
 				g_value_set_pointer( value, self->private->provider );
 				break;
 
+			case NAOBJECT_ITEM_PROP_PROVIDER_DATA_ID:
+				g_value_set_pointer( value, self->private->provider_data );
+				break;
+
 			default:
 				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
 				break;
@@ -284,6 +299,10 @@ instance_set_property( GObject *object, guint property_id, const GValue *value,
 				self->private->provider = g_value_get_pointer( value );
 				break;
 
+			case NAOBJECT_ITEM_PROP_PROVIDER_DATA_ID:
+				self->private->provider_data = g_value_get_pointer( value );
+				break;
+
 			default:
 				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
 				break;
@@ -412,46 +431,18 @@ na_object_item_get_icon( const NAObjectItem *item )
  * Returns the #GdkPixbuf image corresponding to the icon.
  * The image has a size of %GTK_ICON_SIZE_MENU.
  */
-GdkPixbuf *na_object_item_get_pixbuf( const NAObjectItem *item, GtkWidget *widget )
+GdkPixbuf *na_object_item_get_pixbuf( const NAObjectItem *item )
 {
-	static const gchar *thisfn = "na_object_item_get_pixbuf";
-	gchar *iconname;
-	GtkStockItem stock_item;
-	GdkPixbuf* icon = NULL;
-	gint width, height;
-	GError* error = NULL;
+	gchar *icon_name;
+	GdkPixbuf* pixbuf;
 
-	g_return_val_if_fail( NA_IS_OBJECT_ITEM( item ), NULL );
+	icon_name = na_object_item_get_icon( item );
 
-	if( !item->private->dispose_has_run ){
+	pixbuf = na_utils_get_pixbuf( icon_name, GTK_ICON_SIZE_MENU );
 
-		iconname = na_object_item_get_icon( item );
+	g_free( icon_name );
 
-		/* TODO: use the same algorythm than Nautilus to find and
-		 * display an icon
-		 */
-		if( iconname ){
-			if( gtk_stock_lookup( iconname, &stock_item )){
-				icon = gtk_widget_render_icon( widget, iconname, GTK_ICON_SIZE_MENU, NULL );
-
-			} else if( g_file_test( iconname, G_FILE_TEST_EXISTS )
-				   && g_file_test( iconname, G_FILE_TEST_IS_REGULAR )){
-
-				gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
-				icon = gdk_pixbuf_new_from_file_at_size( iconname, width, height, &error );
-				if( error ){
-					g_warning( "%s: iconname=%s, error=%s", thisfn, iconname, error->message );
-					g_error_free( error );
-					error = NULL;
-					icon = NULL;
-				}
-			}
-		}
-
-		g_free( iconname );
-	}
-
-	return( icon );
+	return( pixbuf );
 }
 
 /**
@@ -509,6 +500,26 @@ na_object_item_get_provider( const NAObjectItem *item )
 	return( provider );
 }
 
+/**
+ * na_object_item_get_provider_data:
+ * @item: the #NAObjectItem object to be requested.
+ *
+ * Returns: the provider's specific data.
+ */
+void *
+na_object_item_get_provider_data( const NAObjectItem *item )
+{
+	void *provider_data = NULL;
+
+	g_return_val_if_fail( NA_IS_OBJECT_ITEM( item ), NULL );
+
+	if( !item->private->dispose_has_run ){
+		g_object_get( G_OBJECT( item ), NAOBJECT_ITEM_PROP_PROVIDER_DATA, &provider_data, NULL );
+	}
+
+	return( provider_data );
+}
+
 /**
  * na_object_item_is_enabled:
  * @item: the #NAObjectItem object to be requested.
@@ -736,6 +747,23 @@ na_object_item_set_provider( NAObjectItem *item, const NAIIOProvider *provider )
 	}
 }
 
+/**
+ * na_object_item_set_provider_data:
+ * @item: the #NAObjectItem object to be updated.
+ * @provider_data: a pointer to provider's specific data.
+ *
+ * Sets the I/O provider's data for this #NAObjectItem.
+ */
+void
+na_object_item_set_provider_data( NAObjectItem *item, const void *provider_data )
+{
+	g_return_if_fail( NA_IS_OBJECT_ITEM( item ));
+
+	if( !item->private->dispose_has_run ){
+		g_object_set( G_OBJECT( item ), NAOBJECT_ITEM_PROP_PROVIDER_DATA, provider_data, NULL );
+	}
+}
+
 /**
  * na_object_item_set_enabled:
  * @item: the #NAObjectItem object to be updated.
@@ -1008,11 +1036,12 @@ object_dump( const NAObject *item )
 
 	if( !NA_OBJECT_ITEM( item )->private->dispose_has_run ){
 
-		g_debug( "%s:   tooltip='%s'", thisfn, NA_OBJECT_ITEM( item )->private->tooltip );
-		g_debug( "%s:      icon='%s'", thisfn, NA_OBJECT_ITEM( item )->private->icon );
-		g_debug( "%s:   enabled='%s'", thisfn, NA_OBJECT_ITEM( item )->private->enabled ? "True" : "False" );
-		g_debug( "%s: read-only='%s'", thisfn, NA_OBJECT_ITEM( item )->private->read_only ? "True" : "False" );
-		g_debug( "%s:  provider=%p", thisfn, ( void * ) NA_OBJECT_ITEM( item )->private->provider );
+		g_debug( "%s:       tooltip='%s'", thisfn, NA_OBJECT_ITEM( item )->private->tooltip );
+		g_debug( "%s:          icon='%s'", thisfn, NA_OBJECT_ITEM( item )->private->icon );
+		g_debug( "%s:       enabled='%s'", thisfn, NA_OBJECT_ITEM( item )->private->enabled ? "True" : "False" );
+		g_debug( "%s:     read-only='%s'", thisfn, NA_OBJECT_ITEM( item )->private->read_only ? "True" : "False" );
+		g_debug( "%s:      provider=%p", thisfn, ( void * ) NA_OBJECT_ITEM( item )->private->provider );
+		g_debug( "%s: provider_data=%p", thisfn, ( void * ) NA_OBJECT_ITEM( item )->private->provider_data );
 
 		/* dump subitems */
 		g_debug( "%s: %d subitem(s) at %p",
@@ -1034,6 +1063,7 @@ object_copy( NAObject *target, const NAObject *source )
 	gboolean enabled;
 	gboolean readonly;
 	gpointer provider;
+	gpointer provider_data;
 	GList *subitems, *it;
 
 	g_return_if_fail( NA_IS_OBJECT_ITEM( target ));
@@ -1048,6 +1078,7 @@ object_copy( NAObject *target, const NAObject *source )
 				NAOBJECT_ITEM_PROP_ENABLED, &enabled,
 				NAOBJECT_ITEM_PROP_READONLY, &readonly,
 				NAOBJECT_ITEM_PROP_PROVIDER, &provider,
+				NAOBJECT_ITEM_PROP_PROVIDER_DATA, &provider_data,
 				NULL );
 
 		g_object_set( G_OBJECT( target ),
@@ -1056,6 +1087,7 @@ object_copy( NAObject *target, const NAObject *source )
 				NAOBJECT_ITEM_PROP_ENABLED, enabled,
 				NAOBJECT_ITEM_PROP_READONLY, readonly,
 				NAOBJECT_ITEM_PROP_PROVIDER, provider,
+				NAOBJECT_ITEM_PROP_PROVIDER_DATA, provider_data,
 				NULL );
 
 		g_free( tooltip );
@@ -1075,7 +1107,8 @@ object_copy( NAObject *target, const NAObject *source )
  * @b: the object which has been initially duplicated from @a, and is
  * being checked for modification status.
  *
- * note 1: The provider is not considered as relevant here
+ * note 1: The provider is not considered as relevant here; neither is
+ * the provider_data
  *
  * note 2: as a particular case, this function is not recursive
  * because the equality test will stop as soon as it fails, and we so
